#include "main.h"

void printCelsius(int textsize);
void printCentered(unsigned char *buf, int textsize);
void printRight(unsigned char *buf, int textsize);

int main(void) {

/*********** Variables ************/

	char buf[64];
	/* Ints to store cursorpos */
	int cx = 0;
	int cy = 0;

	/* Ints to store cursorpos for voltage */
	int cxV = 0;
	int cyV = 0;

	/* Ints to store cursorpos for temp */
	int cxT = 0;
	int cyT = 0;

	/* <3 */
	unsigned char heartbeat[2] = {3,0};

	/* Vars to store ADC-readings */
	float therR = 0;
	float TECtemp = 0;
	float Vin = 0;
	float Vpelt = 0;



/************** Startup *************/

	/* Initialize ADC */
	adc_conf();

	/* initialize LCD */
	LCD_init();
	LCD_setOrientation(ORIENTATION_LANDSCAPE);
	LCD_fillScreen(BLACK);
	LCD_setTextBgColor(BLACK);

	/* Print Title */
	LCD_setTextSize(1);
	LCD_setTextColor(GREEN);
	LCD_setCursor(0, 25);
    printCentered((unsigned char *)"The Otter DIY Raman spectrometer",1);

	/* Sensor 1 - TEC-voltage */
	LCD_setTextColor(WHITE);
	LCD_setCursor(20, 60);
    LCD_writeString((unsigned char *)"Peltier voltage = ");

	cx = LCD_getCursorX();
	cy = LCD_getCursorY();
	LCD_setCursor(cx+35,cy);
	LCD_writeString((unsigned char *)"V");

	/* get cursorposition to print voltage */
	cxV = cx+32;
	cyV = cy;


	/* Sensor 1 - TEC-voltage */
	LCD_setCursor(20, 80);
    LCD_writeString((unsigned char *)"CCD temperature = ");

	cx = LCD_getCursorX();
	cy = LCD_getCursorY();
	LCD_setCursor(cx+35,cy);
	printCelsius(1);

	/* get cursorposition to print voltage */
	cxT = cx+32;
	cyT = cy;


	/* Print misc stuff */
	LCD_setCursor(20, 100);
    LCD_writeString((unsigned char *)"STM32F103 SystemCoreClock = ");

	itoa(SystemCoreClock/1000000,buf,10);
    LCD_writeString((unsigned char *)buf);
	LCD_writeString((unsigned char *)" MHz");

	LCD_setCursor(20, 120);
    LCD_writeString((unsigned char *)"STM32F401 SystemCoreClock = 84 MHz");

	LCD_setCursor(20, 140);
    LCD_writeString((unsigned char *)"TCD1304 MCLK = 2.00 MHz");

	LCD_setCursor(20, 160);
    LCD_writeString((unsigned char *)"Raman excitation wavelength = 532 nm");


/*************** The read-loop **************/

    while (1){
		LCD_setTextSize(1);
		LCD_setTextColor(RED);
		LCD_setCursor(265, 25);
		/* Toggle between heart and blank space */
		heartbeat[0] ^= 0x23;
		LCD_writeString(heartbeat);
		delay_ms(500);
	
		
		/* Read ADC1 channel 8 (PB0) and convert to T */
		therR =  10000.0/(4095.0/readADC(8) - 1.0);
		TECtemp = 1.0/298.15 + 1.0/3976.0*log(therR/10000.0); 
		TECtemp = 1.0/TECtemp - 273.15;

	    LCD_fillRect(cxT-36, cyT, 36, 7, BLACK);
		LCD_setCursor(cxT, cyT); 
		/* Print TEC-voltage */
		sprintf(buf,"%.1f",TECtemp);
		printRight((unsigned char *)buf,1);
		

		/*  Read ADC1 channel 9(PB1) and convert to voltage */
		/* The ADC is 12-bit so the voltage on PB1 (Vin) is:
			Vin = 3.3V/(2^12 - 1) * ADC_value   */
		Vin = 3.3*readADC(9) / 4095;
		/* Vin is related to Vpelt like this: 
			Vin = Vpelt*(1 - R1/(R1+R2)) */
		//Vpelt = Vin/0.29;
		Vpelt = Vin/(1.0 - 24.0/34.0);

        LCD_fillRect(cxV-36, cyV, 36, 7, BLACK);
		LCD_setCursor(cxV, cyV);
		/* Print TEC-voltage */
		sprintf(buf,"%.1f",Vpelt);
		printRight((unsigned char *)buf,1);
		   
	}
}



void printCelsius(int textsize){
	/* 	This function prints °C at the current cursor
		position with text size passed. */
	int cx = LCD_getCursorX();
	int cy = LCD_getCursorY();

	/* print degree-sign */
	LCD_setCursor(cx,cy-3*textsize);
	char buf1[2] = {9, 0};
    LCD_writeString((unsigned char *)buf1);

	/* Print C */
	buf1[0]	= 67;
	cx = LCD_getCursorX();
	LCD_setCursor(cx,cy);
    LCD_writeString((unsigned char *)buf1);
} 

void printCentered(unsigned char *buf, int textsize){
	int cy = LCD_getCursorY();

	/* Calculate cursorposition*/
	int textwidth = 6*textsize*strlen((const char*)buf);
	int cx = LCD_getWidth()/2-textwidth/2;
	LCD_setCursor(cx,cy);
    LCD_writeString((unsigned char *)buf);
}


void printRight(unsigned char *buf, int textsize){
	int cx = LCD_getCursorX();
	int cy = LCD_getCursorY();

	/* Calculate cursorposition*/
	int textwidth = 6*textsize*strlen((const char*)buf);
	cx = cx-textwidth;
	LCD_setCursor(cx,cy);
    LCD_writeString((unsigned char *)buf);
}
